@@ -3,12 +3,13 @@
 from bson.objectid import ObjectId
 from tavi import Connection
 from tavi.base.documents import BaseDocument, BaseDocumentMetaClass
+from tavi.commands import Insert, Update
 from tavi.errors import TaviConnectionError
 from tavi.utils.timer import Timer
-import collections
-import datetime
 import inflection
 import logging
+import pymongo
+import re
 
 logger = logging.getLogger(__name__)
 
@@ -50,6 +51,13 @@ class Document(BaseDocument):
     def __init__(self, **kwargs):
         self._id = kwargs.pop("_id", None)
         super(Document, self).__init__(**kwargs)
+        for k, v in self._field_descriptors.items():
+            if v.unique:
+                opts = {
+                    "name": "%s_unique_index" % k,
+                    "unique": True
+                }
+                self.__class__.collection.create_index(k, **opts)
 
     @property
     def bson_id(self):
@@ -166,35 +174,30 @@ class Document(BaseDocument):
         if not self.valid:
             return False
 
-        collection = self.__class__.collection
-        timer = Timer()
-        operation = None
-
         write_opts = frozenset(["w", "j", "wtimeout"])
         kwargs = {k: v for k, v in locals().iteritems() if k in write_opts}
 
-        now = datetime.datetime.utcnow()
-        self.__update_timestamps("last_modified_at", now)
+        operation = Update if self.bson_id else Insert
+        operation = operation(self, **kwargs)
 
-        if self.bson_id:
-            operation = "UPDATE"
-            with timer:
-                result = collection.update(
-                    {"_id": self._id},
-                    {"$set": self.mongo_field_values},
-                    upsert=True,
-                    **kwargs
-                )
+        timer = Timer()
+        with timer:
+            try:
+                operation.execute()
+            except pymongo.errors.PyMongoError as e:
+                operation.reset_fields()
 
-            if result.get("err"):
-                logger.error(result.get("err"))
-
-        else:
-            operation = "INSERT"
-            self.__update_timestamps("created_at", now)
-
-            with timer:
-                self._id = collection.insert(self.mongo_field_values, **kwargs)
+                if isinstance(e, pymongo.errors.DuplicateKeyError):
+                    logger.warn(
+                        "%s %s failed due to unique index violation (%s)",
+                        self.__class__.__name__,
+                        operation.name,
+                        e.message
+                    )
+                    f = re.search(r'\$(.+)_unique_index', e.message).group(1)
+                    self.errors.add(f, "must be unique")
+                    return False
+                raise
 
         self.changed_fields = set()
 
@@ -202,26 +205,12 @@ class Document(BaseDocument):
             "(%ss) %s %s %s, %s",
             timer.duration_in_seconds(),
             self.__class__.__name__,
-            operation,
+            operation.name,
             self.mongo_field_values,
             self._id
         )
         return True
 
-    def __update_timestamps(self, name, timestamp):
-        # TODO: Should this be pulled out into a generic function? i.e.
-        #       set_nested_field_attr(cls, field, value)
-        for field in self.fields:
-            value = getattr(self, field)
-            if name == field:
-                setattr(self, name, timestamp)
-            elif isinstance(value, collections.Iterable):
-                for item in value:
-                    if isinstance(item, EmbeddedDocument):
-                        setattr(item, name, timestamp)
-            elif isinstance(value, EmbeddedDocument):
-                setattr(value, name, timestamp)
-
 
 class EmbeddedDocument(BaseDocument):
     """Represents a single EmbeddedDocument. Supports an *owner* attribute that
